#!/usr/bin/env python

"""
Common scheduling functionality.

Copyright (C) 2015, 2016 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.text import parse_line
from imiptools.handlers.scheduling.manifest import scheduling_functions
import re

def apply_scheduling_functions(functions, handler):

    """
    Apply the given scheduling 'functions' in the current object of the given
    'handler'.
    """

    # Obtain the actual scheduling functions with arguments.

    functions = get_scheduling_function_calls(functions)

    response = "ACCEPTED"

    for fn, args in functions:

        # NOTE: Should signal an error for incorrectly configured resources.

        if not fn:
            return "DECLINED"

        # Keep evaluating scheduling functions, stopping only if one
        # declines or gives a null response.

        else:
            result = fn(handler, args)

            # Return a negative result immediately.

            if not result or result == "DECLINED":
                return result

            # Modify the eventual response from acceptance if a countering
            # result is obtained.

            elif response == "ACCEPTED":
                response = result

    return response

def get_scheduling_function_calls(lines):

    """
    Parse the given 'lines', returning a list of (function, arguments) tuples,
    with each function being a genuine function object and with the arguments
    being a list of strings.

    Each of the 'lines' should employ the function name and argument strings
    separated by whitespace, with any whitespace inside arguments quoted using
    single or double quotes.
    """

    functions = []

    for line in lines:
        parts = parse_line(line)
        functions.append((scheduling_functions.get(parts[0]), parts[1:]))

    return functions

# vim: tabstop=4 expandtab shiftwidth=4
